// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_POSIX_READINGBUFFER_H
#define HU_VARADIISTVAN_SCPL_IO_POSIX_READINGBUFFER_H
//------------------------------------------------------------------------------

#include "StreamBuffer.h"

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * A buffer used for reading.
 *
 * It should be made empty, and then the function read() should be
 * called on it. If it returns true, some data could be read. If the
 * amount of data read is 0, that indicates and end-of-file condition.
 */
class ReadingBuffer : public StreamBuffer
{
protected:
    /**
     * Construct the buffer.
     */
    ReadingBuffer(size_t capacity, BufferedStream* stream);

public:
    /**
     * Read into the buffer using the buffered stream if possible.
     *
     * If the buffer contains data, this function returns immediately
     * with success, so the buffer should be reset before reading.
     *
     * If some data can be read immediately, that is stored in the
     * buffer, and success is indicated. Success is indicated even if
     * the amount of data read is 0, allowing for signalling the
     * end-of-file or end-of-connection condition.
     *
     * If the reading operation would block, the stream is set up so
     * that it would read, and failure is returned.
     *
     * Otherwise, if the reading operation fails for some other
     * reason, the buffer's error code is set, so it should always be
     * checked on a failure!
     */
    bool read();

    friend class BufferedStream;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline ReadingBuffer::ReadingBuffer(size_t capacity, BufferedStream* stream) :
    StreamBuffer(capacity, stream)
{
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_POSIX_READINGBUFFER_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
